@@ -5,7 +5,7 @@ set -x
 
 export PYTHONPATH=./docs/src
 # Use xdist-pytest --forked to ensure modified sys.path to import relative modules in examples keeps working
-pytest --cov=spacy_ann --cov-report=term-missing -o console_output_style=progress --forked --numprocesses=auto ${@}
+pytest --cov=spacy_ann --cov-report=term-missing -o console_output_style=progress
 # bash ./scripts/lint.sh
 # Include tests for files
 bash ./scripts/test-files.sh
